"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceController = void 0;
const uart_1 = require("@espruino-tools/uart");
const fetchHelper_1 = require("./helpers/fetchHelper");
class DeviceController {
    constructor() {
        this.connected = false;
        this.UART = uart_1.uart;
        this.deviceType = undefined;
    }
    /**
     *
     * @returns  promise containing all code stored on device
     */
    dump() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.eval('E.dumpStr()');
        });
    }
    /**
     *
     * @returns device type of connected device
     */
    getDeviceType() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.eval(`process.env.BOARD`);
        });
    }
    /**
     *
     * @returns battery percentage of the connected device
     */
    getBattery() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.eval(`E.getBattery()`);
        });
    }
    write(code) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.eval(code);
        });
    }
    /**
     *
     * @param code code to be evaluated
     * @returns the response from the device in a Promise
     */
    eval(code) {
        return __awaiter(this, void 0, void 0, function* () {
            const p = new Promise((resolve) => {
                let callback = (data) => {
                    resolve(data);
                };
                this.UART.eval(code, callback);
            }).catch((err) => {
                throw new Error(err);
            });
            return p;
        });
    }
    /**
     *
     * @param callback the function to be run after connect
     */
    connect(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.write('\x03;\n').then(() => {
                this.connected = true;
                this.UART.write('digitalPulse(LED2,1,100);\n');
                this.getDeviceType().then((type) => {
                    this.deviceType = type;
                    callback();
                });
            });
        });
    }
    /**
     *
     * @param callback the function to be run after disconnect
     */
    disconnect(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.write('digitalPulse(LED1,1,100);\n').then(() => {
                var _a;
                (_a = this.UART) === null || _a === void 0 ? void 0 : _a.close();
                this.connected = false;
                this.deviceType = undefined;
                callback();
            });
        });
    }
    reset() {
        this.UART.write('reset(true);\n');
    }
    upload(url, flash = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let deviceType = yield this.getDeviceType();
            if (deviceType === 'BANGLEJS') {
                flash = false;
            }
            let success = false;
            yield (0, fetchHelper_1.fetchToText)(url).then((rawCode) => __awaiter(this, void 0, void 0, function* () {
                this.dump().then((response) => __awaiter(this, void 0, void 0, function* () {
                    // check if code is already on the device.
                    // if its already there set success to false,
                    //
                }));
                this.reset();
                if (!flash && !success) {
                    this.UART.write(rawCode);
                }
                else if (!success && deviceType !== 'PIXLJS') {
                    this.UART.write(`E.setBootCode(\`${rawCode}\`,true);\n`);
                    this.UART.write('load();\n');
                }
                else if (!success) {
                    this.UART.write(rawCode);
                    this.UART.write('save();\n');
                    this.UART.write('load();\n');
                }
            }));
        });
    }
}
exports.DeviceController = DeviceController;
