"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Puck = void 0;
const device_controller_1 = require("./device-controller");
const funcToString_1 = require("./helpers/funcToString");
const puck_types_1 = require("./types/puck-types");
class Puck extends device_controller_1.DeviceController {
    constructor() {
        super(...arguments);
        this.pin = {
            val: function () {
                throw new Error('Function not implemented.');
            },
            onHigh: function (pin, func) {
                throw new Error('Function not implemented.');
            },
        };
        this.mag = {
            enable: function () {
                throw new Error('Function not implemented.');
            },
            disabled: function () {
                throw new Error('Function not implemented.');
            },
            onMag: function () {
                throw new Error('Function not implemented.');
            },
            onField: function () {
                throw new Error('Function not implemented.');
            },
        };
        this.accel = {
            enable: function () {
                throw new Error('Function not implemented.');
            },
            disable: function () {
                throw new Error('Function not implemented.');
            },
            val: function () {
                throw new Error('Function not implemented.');
            },
            onAccel: function (func) {
                throw new Error('Function not implemented.');
            },
            onMove: function (func) {
                throw new Error('Function not implemented.');
            },
            onSignificantMove: function (func) {
                throw new Error('Function not implemented.');
            },
            stepCount: {
                enable: function () {
                    throw new Error('Function not implemented.');
                },
                disable: function () {
                    throw new Error('Function not implemented.');
                },
                get: function () {
                    throw new Error('Function not implemented.');
                },
            },
        };
        this.IR = {
            set: function (data) {
                throw new Error('Function not implemented.');
            },
            reset: function () {
                throw new Error('Function not implemented.');
            },
        };
        this.LED = {
            /**
             *
             * @param color LED Colour to be turned on
             */
            on: (color) => {
                Array.isArray(color)
                    ? this.UART.write(`digitalWrite(${color}, 1)`)
                    : this.UART.write(`LED${puck_types_1.LEDColours.indexOf(color) + 1}.set();\n`);
            },
            /**
             *
             * @param color LED Colour to be turned off
             */
            off: (color) => {
                Array.isArray(color)
                    ? this.UART.write(`digitalWrite(${color}, 0)`)
                    : this.UART.write(`LED${puck_types_1.LEDColours.indexOf(color) + 1}.reset();\n`);
            },
            /**
             *
             * @param color LED Colour to be toggled
             */
            toggle: (color) => {
                this.UART.write(`LED${puck_types_1.LEDColours.indexOf(color) + 1}.toggle();\n`);
            },
            /**
             *
             * @param color LED Colour to be flashed
             * @param ms time for LED to be flashed
             */
            flash: (color, ms) => {
                this.UART.write(`digitalPulse(LED${puck_types_1.LEDColours.indexOf(color) + 1},1,${ms});\n`);
            },
            /**
             *
             * @param color LED colour to grab info from
             * @returns a boolean regarding if the LED is on or off
             */
            val: (color) => {
                return this.eval(`digitalRead(LED${puck_types_1.LEDColours.indexOf(color) + 1}) == 1`);
            },
        };
        this.NFC = {
            /**
             *
             * @param url the url to be used as the new NFC value
             */
            setUrl: (url) => this.UART.write('NRF.nfcURL("' + url + '");\n'),
            reset: () => this.UART.write('NRF.nfcURL();\n'),
        };
    }
    getLightVal() {
        throw new Error('Method not implemented.');
    }
    /**
     *
     * @returns temperature from device
     */
    getTemperature() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.eval(`E.getTemperature()`);
        });
    }
    /**
     *
     * @param func A function to be run of press of pucks button
     */
    onPress(func) {
        this.UART.write(`
    setWatch(function(){
      ${(0, funcToString_1.stringifyFunction)(func)};
    }, BTN,{edge:"rising", repeat:true, debounce:50})
  `);
    }
    /**
     *
     * @param long The function to be called on a long press
     * @param short The function to be called on a short press
     * @param ms the time required to consider a press a long press
     */
    onTimedPress(long, short, ms = 0.3) {
        this.UART.write(`
      setWatch(function(){
        var ms = (e.time = e.lastTime);

        if(ms > ${ms}){
          ${(0, funcToString_1.stringifyFunction)(long)};
        } else {
          ${(0, funcToString_1.stringifyFunction)(short)};
        }
      }, BTN,{edge:"falling", repeat:true, debounce:50})
    `);
    }
}
exports.Puck = Puck;
